import pathlib as pl

MANIFEST_FILE_NAME = "manifest.yml"

MANIFEST_DEFAULT_NAME = "my-monorepo"
MANIFEST_DEFAULT_RELATIVE_PATH = pl.Path("src")
MANIFEST_DEFAULT_DESCRIPTION = "A monorepo for my project"
